/**
 * Returns middleware for ./index.js Server application instance
 *
 * @async
 * @param {{}} cfg Config
 * @returns {Promise} Middleware for ./index.js
 */

const querystring = require('querystring');

module.exports = (cfg) => async (ctx) => {
  const {
    config, logger, reqError
  } = ctx.state;

  if (config.requestAuthorised) {
    // FIXME: Надо передавать нужные классы в cfg.classes вместо подключения файлов в этом модуле из другого проекта.
    // const User = require(cfg.userClassPath || '../../classes/User');
    // const Device = require(cfg.deviceClassPath || '../../classes/Device');
    const User = cfg.classes && cfg.classes.User ? cfg.classes.User : require(cfg.userClassPath || '../../classes/User');
    const Device = cfg.classes && cfg.classes.Device ? cfg.classes.Device : require(cfg.userClassPath || '../../classes/Device');

    try {
      ctx.state.user = config.guest;
      const deviceIDFromHeaders = ctx.get('device-id');

      if (deviceIDFromHeaders) {
        if (deviceIDFromHeaders === 'unknown') {
          reqError.throw(new Error('Unknown device'), 500, ctx.headers);
        }

        ctx.state.device = (deviceIDFromHeaders === config.guest.ID) ? Device.guest : await Device.get({ ID: deviceIDFromHeaders }, ctx.state);
      } else {
        ctx.state.device = Device.create(JSON.parse(querystring.unescape(ctx.get('auth_device'))), ctx.state);
      }
    } catch (err) {
      reqError.throw(err, 500, ctx.headers);
    }

    try {
      const userIDFromHeaders = ctx.get('user-id');

      if (userIDFromHeaders) {
        if (userIDFromHeaders === 'unknown') {
          reqError.throw(new Error('Unknown user'), 500, ctx.headers);
        }

        ctx.state.user = ctx.state.device.isGuest ? User.guest : await User.get({ ID: userIDFromHeaders }, ctx.state);
      } else {
        ctx.state.user = User.create(JSON.parse(querystring.unescape(ctx.get('auth_user'))), ctx.state);
      }
    } catch (err) {
      reqError.throw(err, 500, ctx.headers);
    }

    if (!ctx.state.user || !ctx.state.device) {
      reqError.throw('Auth FAIL', 500, ctx.headers);
    }

    ctx.state.user.device = ctx.state.device;

    // TODO: Пробросить через хедер из ауфа
    // try {
    //     ctx.state.user.activeACLs  = Device.create( JSON.parse(decodeURIComponent( ctx.get( 'auth_acls' ) )), ctx.state );
    // } catch (err) {
    //     reqError.throw(err, 500, ctx.headers );
    // }

    await ctx.state.user.authInit();

    // 2. Получаем геопозичию запроса
    ctx.state.device.geo = querystring.unescape(ctx.get('rG'));
    ctx.state.device.log.debug('Update geo', ctx.state.device.geo);

    logger.debug('Device and User init...');
  }
};
