const asyncSleep = (s) => new Promise((resolve) => setTimeout(resolve, s * 1000));

const querystring = require('querystring');

const Logger = require('./Logger');

const keyMap = {
  rD: 'd',
  rKey: 'k',
};

/**
 * Update logger env variables by request headers
 * @param {{}} ctx koa context
 * @param {{}} logger logger instance
 */
const updateLoggerEnvByReqCtx = (ctx, logger) => {
  for (const key of Logger.reqInfoKeys) {
    const val = ctx.get(key) || ctx.query[key] || ctx.query[keyMap[key]];

    if (val || val === 0) {
      logger.setEnv(key, querystring.unescape(val));
    }
  }

  if (!logger.getEnv('rKey')) {
    logger.setEnv('rKey', `${Date.now() / 1000}`);
  }

  const loggerRKey = logger.getEnv('rKey');

  if (loggerRKey.length < 32) {
    // Значит у нас там только TS и надо добавить guid
    logger.setEnv('rKey', `${loggerRKey}-${String.generateGUID()}`);
  }
  // update request eKey header
  // todo убрать отсюда

  ctx.set('rKey', logger.getEnv('rKey'));

  // set client ip to logger
  const xRealIpHeader = ctx.get('rIP');

  if (xRealIpHeader) {
    logger.setEnv('rIP', xRealIpHeader);
  }
};

/**
 * Update response headers by logger env variables
 * @param {{}} ctx koa context
 * @param {{}} logger logger instance
 */
const updateResHeadersByLoggerEnv = (ctx, logger) => {
  for (const key of Logger.reqInfoKeys) {
    const val = logger.getEnv(key);

    if (val || val === 0) {
      ctx.set(key, querystring.escape(val));
    }
  }
};

const updateLoggerEnvByDeviceFields = (deviceFields, logger, { force = false } = {}) => {
  const map = {
    rDp: 'devPlatform',
    rD: 'ID',
    rDn: 'devName',
    rU: 'userID',
    rCa: 'appName',
    rCv: 'appVer',
  };

  for (const [envName, deviceFieldName] of Object.entries(map)) {
    const deviceField = deviceFields[deviceFieldName];

    if (force || (!logger.getEnv(envName) && deviceField !== null && deviceField !== undefined)) {
      logger.setEnv(envName, deviceField);
    }
  }
};

module.exports = {
  asyncSleep,
  updateResHeadersByLoggerEnv,
  updateLoggerEnvByReqCtx,
  updateLoggerEnvByDeviceFields,
};
